<?php
/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\DiscountRule\Model;


use Magento\Framework\Model\AbstractModel;
use Magento\Quote\Api\Data\TotalsInterface;
use Magento\SalesRule\Model\Rule\Condition\Address as SalesRuleAddress;

class Address extends \Magento\SalesRule\Model\Rule\Condition\Address
{
    const KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING = 'subtotal_with_discount_exclude_shipping';
    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'base_subtotal' => __('Subtotal'),
            'total_qty' => __('Total Items Quantity'),
            'weight' => __('Total Weight'),
            'shipping_method' => __('Shipping Method'),
            'postcode' => __('Shipping Postcode'),
            'region' => __('Shipping Region'),
            'region_id' => __('Shipping State/Province'),
            'country_id' => __('Shipping Country'),
        ];
        $attributes[TotalsInterface::KEY_BASE_SUBTOTAL_INCL_TAX] = __('Subtotal (Including Tax)');
        $attributes[TotalsInterface::KEY_BASE_GRAND_TOTAL] = __('Base Grand Total');
        $attributes[TotalsInterface::KEY_SUBTOTAL_WITH_DISCOUNT] = __('Subtotal With Discount');
        $attributes[TotalsInterface::KEY_BASE_SUBTOTAL_WITH_DISCOUNT] = __('Base Subtotal With Discount');
        $attributes[TotalsInterface::KEY_GRAND_TOTAL] = __('Grand Total');
        $attributes[self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING] = __('Subtotal with discounts no shipping (Incl. Tax)');

        $this->setAttributeOption($attributes);

        return $this;
    }


    /**
     * Get input type
     *
     * @return string
     */
    public function getInputType()
    {
        switch ($this->getAttribute()) {
            case TotalsInterface::KEY_BASE_SUBTOTAL_INCL_TAX:
            case TotalsInterface::KEY_SUBTOTAL_WITH_DISCOUNT:
            case TotalsInterface::KEY_BASE_SUBTOTAL_WITH_DISCOUNT:
            case TotalsInterface::KEY_GRAND_TOTAL:
            case self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING:
            case 'base_subtotal':
            case 'base_grand_total':
            case 'weight':
            case 'total_qty':
                return 'numeric';

            case 'shipping_method':
            case 'payment_method':
            case 'country_id':
            case 'region_id':
                return 'select';
        }
        return 'string';


    }


    /**
     * Get value select options
     *
     * @return array|mixed
     */
    public function getValueSelectOptions()
    {
        if (!$this->hasData('value_select_options')) {
            switch ($this->getAttribute()) {
                case 'country_id':
                    $options = $this->_directoryCountry->toOptionArray();
                    break;

                case 'region_id':
                    $options = $this->_directoryAllregion->toOptionArray();
                    break;

                case 'shipping_method':
                    $options = $this->_shippingAllmethods->toOptionArray();
                    break;

                case 'payment_method':
                    $options = $this->_paymentAllmethods->toOptionArray();
                    break;

                default:
                    $options = [];
            }
            $this->setData('value_select_options', $options);
        }

    }

    /**
     * Validate Address Rule Condition
     *
     * @param \Magento\Framework\Model\AbstractModel $model
     * @return bool
     */
    public function validate(\Magento\Framework\Model\AbstractModel $model)
    {
        $address = $model;
        if (!$address instanceof \Magento\Quote\Model\Quote\Address) {
            if ($model->getQuote()->isVirtual()) {
                $address = $model->getQuote()->getBillingAddress();
            } else {
                $address = $model->getQuote()->getShippingAddress();
            }
        }

        if ('payment_method' == $this->getAttribute() && !$address->hasPaymentMethod()) {
            $address->setPaymentMethod($model->getQuote()->getPayment()->getMethod());

        }
        if (self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING !== $this->getAttribute()) {
            return  null;
        }

        $address->setData(self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING, $address->getSubtotalInclTax() + $address->getDiscountAmount());
        $address->setBaseSubtotalWithDiscount($address->getBaseSubtotal() + $address->getDiscountAmount());

        return parent::validate($address);
    }
}
